/** @file truncate.c
 * Truncating of files
 * @author Marko Mäkelä <marko.makela@iki.fi>
 */

/* Copyright © 2006 Marko Mäkelä.

   This file is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#if defined WIN32 || defined __WIN32
# undef __STRICT_ANSI__ /* for fileno() */
# include <windows.h>
# include <io.h>
#else
# define _POSIX_SOURCE
# ifndef _POSIX_C_SOURCE
#  define _POSIX_C_SOURCE 200112L /* for ftruncate(2) */
# endif
# include <unistd.h>
# include <sys/types.h>
#endif
#include <stdio.h>

/** Truncate a file at its current position
 * @param file	file handle
 * @return	zero on success, nonzero on error
 */
int
truncate_file (FILE* f)
{
#if defined WIN32 || defined __WIN32
  if (!SetEndOfFile ((HANDLE) _get_osfhandle (fileno (f)))) {
    char* msg;
    long err;
    err = GetLastError ();
    FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER |
		   FORMAT_MESSAGE_FROM_SYSTEM |
		   FORMAT_MESSAGE_IGNORE_INSERTS,
		   0,
		   err,
		   0, /* default language */
		   (LPTSTR) &msg,
		   0,
		   0);
    AnsiToOem (msg, msg);
    (void) fprintf (stderr, "SetEndOfFile failed with code %ld: %s\n",
		    err, msg);
    LocalFree (msg);
    return -1;
  }
#else
  if (ftruncate (fileno (f), ftell (f))) {
    perror ("ftruncate");
    return -1;
  }
#endif
  return 0;
}
